"""
Synbiopython (c) Global BioFoundry Alliance 2020

Synbiopython is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author: neilswainston
"""
import os.path

DATA_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data")
